import React, { useEffect, useState, useContext, createContext } from 'react';
import * as Notifications from 'expo-notifications';
import { storage, StoreType } from 'src/storage';
import { Linking, Platform } from 'react-native';
import { useNavigation } from '@react-navigation/native';

const PushNotificationContext = createContext(null);

export const usePushNotification = () => useContext(PushNotificationContext);

export const PushNotificationProvider = ({ children }) => {
  const [isSubscribed, setIsSubscribed] = useState(
    (storage.get('subscribed', StoreType.BOOLEAN) as boolean) ?? false
  );
  const navigation = useNavigation();

  const lastNotificationResponse = Notifications.useLastNotificationResponse();

  useEffect(() => {
    if (lastNotificationResponse && Platform.OS === 'android') {
      console.log(
        'lastNotificationResponse',
        lastNotificationResponse.notification.request.content.data
      );
      const data = lastNotificationResponse.notification.request.content.data;

      if (data?.screen) {
        navigation.navigate(data.screen as never);
      }
      if (data?.url) {
        Linking.openURL(data.url);
      }
    }
  }, [lastNotificationResponse]);

  useEffect(() => {
    if (isSubscribed) {
      const notificationListener = Notifications.addNotificationReceivedListener((notification) => {
        console.log('Notification received', notification.request);
      });

      const responseListener = Notifications.addNotificationResponseReceivedListener((response) => {
        console.log('Notification response received', response.notification.request);

        let screenName;
        let url;
        if (Platform.OS === 'ios') {
          console.log('data ios', response.notification.request.trigger?.payload);
          screenName = response.notification.request.trigger?.payload?.screen;
          url = response.notification.request.trigger?.payload?.url;
        }

        if (screenName) {
          navigation.navigate(screenName as never);
        }
        if (url) {
          Linking.openURL(url);
        }
      });

      return () => {
        notificationListener.remove();
        responseListener.remove();
      };
    }
  }, [isSubscribed]);

  const subscribeToNotifications = async () => {
    storage.set('subscribed', true);
    setIsSubscribed(true);
  };

  const unsubscribeFromNotifications = async () => {
    await removeNotificationTokenFromServer();
    storage.remove('deviceToken');
    storage.set('subscribed', false);
    setIsSubscribed(false);
  };

  const toggleSubscription = async () => {
    if (isSubscribed) {
      await unsubscribeFromNotifications();
    } else {
      await subscribeToNotifications();
    }
  };

  return (
    <PushNotificationContext.Provider value={{ isSubscribed, toggleSubscription }}>
      {children}
    </PushNotificationContext.Provider>
  );
};

async function removeNotificationTokenFromServer() {}
